
//  Copyright 2021 Aaron Hance
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

{ 
	Drivers->Storage->AHCI->AHCI - AHCI Driver.
	
	@author(Aaron Hance <ah@aaronhance.me>)
}
unit AHCI;

interface

uses 
    util,
    PCI,
    drivertypes,
    drivermanagement,
    lmemorymanager,
    console,
    vmemorymanager,
    AHCITypes,
    lists;

var
    ahciControllers : PDLList;

procedure init();
procedure load();

implementation

procedure init();
var
    devID : TDeviceIdentifier;
begin
    console.writestringln('AHCI: Registering driver');
    devID.bus:= biPCI;
    devID.id0:= idANY;
    devID.id1:= $00000001;
    devID.id2:= $00000006;
    devID.id3:= $00000001;
    devID.id4:= idANY;
    devID.ex:= nil;
    drivermanagement.register_driver('ATA/PI AHCI Driver', @devID, @load);
    //TODO check IDE devices in ide for sata devices
end;

procedure check_ports(controller : PAHCI_Controller);
var
    i : uint32;
    port : PHBA_Port;
begin
end;

procedure load(ptr : void);
var 
    device : PPCI_Device;
    controller : PAHCI_Controller;
    i : uint32;
    base : PHBA_Memory;
begin
    console.writestringln('AHCI: initilizing a new controller');

    if ahciControllers = nil then begin
        console.writestringln('AHCI: Initializing controller list');
        ahciControllers := DL_New(SizeOf(TAHCI_Controller));
    end;

    device := PPCI_Device(ptr);

    controller := DL_Add(ahciControllers);
    controller^.device := device;

    base: PHBA_Memory(kpalloc(device^.address5)); // get the base address of the controller
    controller^.mio := base;

    //enable AHCI mode, TODO check if is not already in AHCI mode and enable it, also perhaps remove if loaded as IDE
    base^.ghc := base^.ghc or AHCI_CONTROLLER_MODE;

    //clear any pending interrupts
    base^.int_status := $FFFFFFFF;





    // console.writestringln('AHCI: initilizing a new controller');
    // ahciControllers[ahciControllerCount] := ptr;
    // hba[ahciControllerCount] := PPCI_Device(ptr)^.address5;
    // new_page_at_address(hba[ahciControllerCount]);

    // //here would be any controller setup needed

    // check_ports(ahciControllerCount);
    // ahciControllerCount += 1;
    // exit(true);
end;


end.
